(function($) {

    var utils   =   {
        'stripHash':    function(hash) {
            return String(hash).replace(/^#!?/, '');
        }
    }
    
    var protos      =   {},
        protoElems  =   ['a', 'section', 'ol', 'li'];
    for (var i = protoElems.length - 1; i >= 0; i--) {
        protos[protoElems[i]]   =   $('<' + protoElems[i] + ' />');
    };
    delete(protoElems);
    
    var body            =   $('body'),
        title           =   $('title'),
        origTitle       =   title.html(),
        slides          =   body.find('section');
    
    $(document).ready(function() {
        $('a[rel*=external]').attr({'target': '_blank'});

        var expose      =   protos.section.clone().attr('id', 'slides').appendTo(body),
            exposeList  =   protos.ol.clone().appendTo(expose);
        exposeList.wrap('<div class="entry-content" />');
        expose.prepend('<header><h1>Slides</h1></header>');
        
        slides.each(function(index) {
            var slide   =   $(this),
                header  =   slide.find('header'),
                next    =   slide.next('section'),
                prev    =   slide.prev('section');
            if(next.size() && !header.find('a[rel=next]').size()) {
                var nextTitle   =   next.find('header h1');
                header.append(protos.a.clone().attr({'href': '#' + next.attr('id'), 'title': nextTitle.text(), 'rel': 'next'}).html(nextTitle.html()));
            }
            if(prev.size() && !header.find('a[rel=prev]').size()) {
                var prevTitle   =   prev.find('header h1');
                header.prepend(protos.a.clone().attr({'href': '#' + prev.attr('id'), 'title': prevTitle.text(), 'rel': 'prev'}).html(prevTitle.html()));
            }
            protos.li.clone().appendTo(exposeList).append(protos.a.clone().attr({'href': '#' + slide.attr('id'), 'title': header.find('h1').text()}).html(header.find('h1').html()));
        });
    });
    
    $(document).bind('keyup', 'home', function() {
        window.location.hash    =   slides.first().attr('id');
    });
    $(document).bind('keyup', 'end', function() {
        window.location.hash    =   slides.last().attr('id');
    });
    function nextSlide() {
        var currentSlide        =   $('section[id="' + utils.stripHash(window.location.hash) + '"]'),
            toHash              =   currentSlide.find('header a[rel=next]').size() ? currentSlide.find('header a[rel=next]').attr('href') : currentSlide.next('section').attr('id');
        if(toHash) {
            window.location.hash    =   toHash;
        }
    }
    function prevSlide() {
        window.history.go(-1);
    }
    $(document).bind('keyup', 'right down enter return', nextSlide);
    $(document).bind('swipeleft', nextSlide);
    $(document).bind('keyup', 'left up', prevSlide);
    $(document).bind('swiperight', prevSlide);
    $(document).bind('keydown', 'esc', function() {
        var hash    =   utils.stripHash(window.location.hash);
        if(hash == 'slides') {
            window.location.hash    =   body.data('oldHash') || slides.first().attr('id');
        } else {
            body.data('oldHash', window.location.hash);
            window.location.hash    =   'slides';
        }
    });
    
    function updateTitle() {
        var currentSlide        =   $('section[id="' + utils.stripHash(window.location.hash) + '"]');
        if(currentSlide.size()) {
            title.html(currentSlide.find('header h1').text() + ': Slide ' + (currentSlide.prevAll('section').size() + 1) + ' of ' + slides.size());
        } else {
            title.html(origTitle + ': Slide 1 of ' + slides.size());
        }
    }
    
    $(window).bind('hashchange', updateTitle);
    updateTitle();

    if(Modernizr && !Modernizr.targetselector) {
        $(window).bind('hashchange', function() {
            sections.removeClass('target');
            sections.filter('[id="' + utils.stripHash(window.location.hash) + '"]').addClass('target');
        });
    }
    
    function toggleColorExamples(c) {
        c.preventDefault();
        var examples    =   $('#color ul[rel=example] li code');
        if(examples.data('toggled')) {
            examples.css({
                'color': ''
            }).data('toggled', false);
        } else {
            examples.each(function() {
                $(this).css({
                    'color': $(this).text()
                });
            }).data('toggled', true);
        }
    }
    
    var colorExampleToggle  =   protos.a.clone().attr({'href': '#toggle-example-colors', 'title': 'Toggle Example Colors'}).text('Toggle Example Colors').addClass('noprint');
    $('#color ul[rel=example]').prevAll('h2').first().append(colorExampleToggle);
    colorExampleToggle.click(toggleColorExamples);
    
    var animationExampleTimeout;
    
    function animationExample(c) {
        c.preventDefault();
        clearTimeout(animationExampleTimeout);
        var example =   $('#css-animations pre');
        example.addClass('highlight');
        setTimeout(function() {
            example.removeClass('highlight');
        }, 3000);
    }
    
    var animationExampleToggle  =   protos.a.clone().attr({'href': '#highlight', 'title': 'Demonstrate Highlight Animation'}).text('Demonstrate').addClass('noprint');
    $('#css-animations h2').append(animationExampleToggle);
    animationExampleToggle.click(animationExample);
    
    if (window.addEventListener) {
      window.addEventListener('pageshow', (function(e) {
        if (e.persisted) {
          return setTimeout((function() {
            return window.scrollTo(window.scrollX, window.scrollY + 1);
          }), 10);
        }
      }), false);
    }
    
})(jQuery);